
/*
 *  wmessage.app -- panel.c [panel]
 *
 *  Copyright (c) 1999,2000 Craig Nellist <crn@ozemail.com.au>
 *  Copyright (c) 1998,1999 Alfredo K. Kojima <kojima@inf.ufrgs.br>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 *  USA.
 */

#include "wmessage.h"

static void clickPanel (WMWidget *self, void *clientData)
{
	messagePanel *panel = (messagePanel *)clientData;
	char *tmp = NULL;

	if (settings->nomouse == True && settings->source == SOURCE_MOUSE)
		return;

	panel->done = 1;
	if (self == panel->defBtn) {
		panel->result = WAPRDefault;
		tmp = settings->def;
	} else if (self == panel->othBtn) {
		panel->result = WAPROther;
		tmp = settings->other;
	} else if (self == panel->altBtn) {
		panel->result = WAPRAlternate;
		tmp = settings->alt;
	} else if (settings->nobuttons == True)
		panel->result = WAPRDefault;

	if (settings->print)
		fprintf (stdout, "%s\n", tmp);

	settings->source = SOURCE_MOUSE;
}

void setDefaultButton (WMButton *btn)
{
	WMSetButtonImage (btn, WMGetSystemPixmap (WMWidgetScreen (btn), WSIReturnArrow));
	WMSetButtonAltImage (btn, WMGetSystemPixmap (WMWidgetScreen (btn), WSIHighlightedReturnArrow));
	WMSetButtonImagePosition (btn, WIPRight);
}

void clearButtonImage (WMButton *btn)
{
	WMSetButtonImage (btn, NULL);
	WMSetButtonAltImage (btn, NULL);
}

static void pressPanel(XEvent *event, void *clientData)
{
	messagePanel *panel = (messagePanel *)clientData;

	settings->source = SOURCE_KEYB;

	if ((event->xkey.keycode == panel->retKey || event->xkey.keycode == panel->entKey) && panel->defBtn) {
		WMPerformButtonClick(panel->defBtn);
		return;
	} else if (event->xkey.keycode == panel->escKey) {
		if (panel->altBtn || panel->othBtn) {
			WMPerformButtonClick(panel->othBtn ? panel->othBtn : panel->altBtn);
		} else {
			panel->result = WAPRDefault;
			panel->done=1;
		}
		return;
	} else {	/* keys besides return/escape */
		KeySym ksym;
		int count, def, alt, other;
		char buffer[8];

		def = alt = other = False;
		count = XLookupString (&event->xkey, buffer, 7, &ksym, NULL);
		buffer[count] = '\0';

		switch (ksym) {
			default:
				if (settings->def)
					if (tolower(*(settings->def)) == tolower(buffer[0]))
						def = True;
				if (settings->alt)
					if (tolower(*(settings->alt)) == tolower(buffer[0]))
						alt = True;
				if (settings->other)
					if (tolower(*(settings->other)) == tolower(buffer[0]))
						other = True;
				if (def && !alt && !other)
					WMPerformButtonClick (panel->defBtn);
				else if (!def && alt && !other)
					WMPerformButtonClick (panel->altBtn);
				else if (!def && !alt && other)
					WMPerformButtonClick (panel->othBtn);
		}
	}
}

static void releasePanel(XEvent *event, void *clientData)
{
	messagePanel *panel = (messagePanel *)clientData;

	clickPanel (panel->win, panel);
}

int showPanel (WMScreen *scrPtr, _settings *settings)
{
	messagePanel *panel;
	int tmp;

	panel = createPanel (scrPtr, settings);

	if (panel == NULL)
		return (-2);

	if (settings->centre == True) {
		WMSetWindowInitialPosition (panel->win,
			(WidthOfScreen (DefaultScreenOfDisplay (settings->dpy)) - WMWidgetWidth(panel->win))/2,
			(HeightOfScreen (DefaultScreenOfDisplay (settings->dpy)) - WMWidgetHeight(panel->win))/2);

		WMMoveWidget (panel->win,
			(WidthOfScreen (DefaultScreenOfDisplay (settings->dpy)) - WMWidgetWidth(panel->win))/2,
			(HeightOfScreen (DefaultScreenOfDisplay (settings->dpy)) - WMWidgetHeight(panel->win))/2);

		WMSetWindowAspectRatio (panel->win, WMWidgetWidth (panel->win), WMWidgetHeight (panel->win), WMWidgetWidth (panel->win), WMWidgetHeight (panel->win));
	} else if (settings->nearmouse == True) {
		int x, y, x2, y2;
		unsigned int mask;
		Window root, child;

		XQueryPointer (settings->dpy, DefaultRootWindow (settings->dpy), &root, &child, &x, &y, &x2, &y2, &mask);

		WMSetWindowInitialPosition (panel->win, x - (WMWidgetWidth(panel->win)/2), y - (WMWidgetHeight(panel->win)/2));
		WMMoveWidget (panel->win, x - (WMWidgetWidth(panel->win)/2), y - (WMWidgetHeight(panel->win)/2));
	}

	WMRealizeWidget (panel->win);

	WMMapWidget (panel->win);

	while (!panel->done || WMScreenPending (scrPtr)) {
		XEvent event;

		WMNextEvent(WMScreenDisplay (scrPtr), &event);
		WMHandleEvent(&event);
	}
	tmp = panel->result;
	WMDeleteTimerHandler (settings->timerTimeout);
	WMDeleteTimerHandler (settings->timerReload);
	destroyPanel (panel);

	return tmp;
}

void destroyPanel (messagePanel *panel)
{
	WMUnmapWidget(panel->win);
	WMDestroyWidget(panel->win);
	free(panel);
}

char * readFromFile (messagePanel *panel, char *filename)
{
	FILE *f;
	struct stat sbuf;
	char *data;
	int count;

	if (stat(filename, &sbuf) < 0 || sbuf.st_size < 0) {
		perror (filename);
		return NULL;
	}

	data = malloc (sbuf.st_size + 2);
	if (!data) {
		fprintf (stderr, "Can't allocate enough memory for file.\n");
		return NULL;
	}

	if ((f = fopen (filename, "r")) == NULL) {
		perror (filename);
		free (data);
		return NULL;
	}

	count = fread (data, 1, sbuf.st_size, f);
	if (!count && sbuf.st_size) {
		perror (filename);
		fclose (f);
		free (data);
		return NULL;
	}

	if (count)
		data[count++] = '\n';

	data[count] = '\0';
	fclose (f);
	return data;
}

char * readFromStdin (messagePanel *panel)
{
	char tmpname[80];
	int tmpfile, i;
	char buf[BUFSIZ];
	char *data;

	strcpy (tmpname, "/tmp/wmessage.XXXXXX");	/* last 6 chars must be X */
	mktemp (tmpname);

	if (!tmpname)
		return NULL;	/* couldn't generate unique name */

	tmpfile = creat(tmpname, 0600);
	if (tmpfile == -1)
		return NULL;	/* couldn't create file */

	while ((i = fread (buf, 1, BUFSIZ, stdin)) > 0) {
		write (tmpfile, buf, i);
	}

	close (tmpfile);
	data = readFromFile (panel, tmpname);
	unlink (tmpname);
	return (data);
}

char * readFile (messagePanel *panel, _settings *settings)
{
	char *msgstr;

	if ((*(settings->filename)) == '-' && (*(settings->filename+1)) == '\0')
		msgstr = readFromStdin (panel);
	else
		msgstr = readFromFile (panel, settings->filename);

	if (msgstr)
		return msgstr;
	else
		return NULL;
}

void reloadCB (void *clientData)
{
	messagePanel *panel = (messagePanel *)clientData;
	char *msg;
	int i, j=0, lines=0, longest=0;

	msg = readFile (panel, settings);
	if (msg == NULL) {
		if (settings->force == False) {
			msg = wstrdup("Cannot open file -- not reloading.\n");
			lines++;
			longest = strlen (msg);
		} else {
			msg = wstrdup ("Cannot open file -- will keep trying.\n");
			lines++;
			longest = strlen (msg);
			WMAddTimerHandler (settings->reload * 1000, reloadCB, panel);
		}
		WMResizeWidget (panel->mLbl, WMWidthOfString(WMSystemFontOfSize (WMWidgetScreen (panel->win), 12), "-", 1)*longest, WMFontHeight(WMSystemFontOfSize (WMWidgetScreen (panel->win), 12))*lines);
		WMSetLabelText (panel->mLbl, msg);
		return;
	}

	for (i = 0; i < strlen (msg); i++) /* count lines */
		if ((*(msg+i)) == '\n') {
			lines++;
			if (longest < j)
				longest = j;
			j=0;
		} else
			j++;

	if (j > longest)
		longest = j;

	if (strlen(msg)) {
		WMResizeWidget (panel->mLbl, WMWidthOfString(WMSystemFontOfSize (WMWidgetScreen (panel->win), 12), "-", 1)*longest, WMFontHeight(WMSystemFontOfSize (WMWidgetScreen (panel->win), 12))*lines);
		WMSetLabelText (panel->mLbl, msg);
	} else {
		WMResizeWidget (panel->mLbl, WMWidgetWidth (panel->view), WMFontHeight(WMSystemFontOfSize (WMWidgetScreen (panel->win), 12)));
		if (settings->emptyfile) {
			char buf[100];

			sprintf (buf, "File '%s' is zero bytes.%c", settings->filename, '\0');
			WMSetLabelText (panel->mLbl, buf);
		}
	}

	if (msg != NULL)
		free (msg);

	WMAddTimerHandler (settings->reload * 1000, reloadCB, panel);
}

void timeoutCB (void *clientData)
{
	messagePanel *panel = (messagePanel *)clientData;

	if (panel->defBtn) {
		WMPerformButtonClick (panel->defBtn);
	} else {
		panel->done = 1;
		panel->result = WAPRDefault;
	}
	WMUnmapWidget (panel->win);
}

messagePanel * createPanel (WMScreen *scrPtr, _settings *settings)
{
	messagePanel *panel;
	WMSize psize;
	WMPixmap *appicon;
	int x, dw=0, aw=0, ow=0, w;

	panel = wmalloc(sizeof(messagePanel));
	memset(panel, 0, sizeof(messagePanel));

	panel->retKey = XKeysymToKeycode(WMScreenDisplay(scrPtr), XK_Return);
	panel->entKey = XKeysymToKeycode(WMScreenDisplay(scrPtr), XK_KP_Enter);
	panel->escKey = XKeysymToKeycode(WMScreenDisplay(scrPtr), XK_Escape);

	panel->win = WMCreateWindowWithStyle(scrPtr, "alertPanel", WMTitledWindowMask);

	if (settings->filename || settings->prompt) {
		if (settings->filename) {
			char *msg;
			int i, j=0, lines=0, longest=0;

			msg = readFile (panel, settings);
			if (msg == NULL)
				if ((settings->reload > 0) && (settings->force == True))
					msg = wstrdup ("Cannot open file -- will keep trying.\n");
				else
					return NULL;

			for (i = 0; i < strlen (msg); i++) /* count lines */
				if ((*(msg+i)) == '\n') {
					lines++;
					if (longest < j)
						longest = j;
					j=0;
				} else
					j++;

			if (j > longest)
				longest = j;

			WMResizeWidget (panel->win, 400, 348);

			panel->view = WMCreateScrollView (panel->win);
			WMResizeWidget (panel->view, 380, 200);
			WMMoveWidget (panel->view, 10, 103);
			WMSetScrollViewRelief (panel->view, WRSunken);
			WMSetScrollViewHasVerticalScroller (panel->view, True);
			WMSetScrollViewHasHorizontalScroller (panel->view, True);

			panel->mLbl = WMCreateLabel (panel->win);
			WMMoveWidget (panel->mLbl, 10, 83);
			if (strlen(msg)) {
				WMResizeWidget (panel->mLbl, WMWidthOfString(WMSystemFontOfSize (scrPtr, 12), "-", 1)*longest, WMFontHeight(WMSystemFontOfSize (scrPtr, 12))*lines);
				WMSetLabelText (panel->mLbl, msg);
			} else {
				WMResizeWidget (panel->mLbl, WMWidgetWidth (panel->view), WMFontHeight(WMSystemFontOfSize (scrPtr, 12)));
				if (settings->emptyfile) {
					char buf[100];

					sprintf (buf, "File '%s' is zero bytes.%c", settings->filename, '\0');
					WMSetLabelText (panel->mLbl, buf);
				}
			}

			WMSetLabelTextAlignment (panel->mLbl, WALeft);
			if (settings->fixedfont == True) {
				WMFont *ffont;

				ffont = WMCreateNormalFont (WMWidgetScreen (panel->win), FIXED_FONT);
				WMSetLabelFont (panel->mLbl, ffont);
			}

			WMSetScrollViewContentView (panel->view, WMWidgetView(panel->mLbl));

			if (msg != NULL)
				free (msg);
		} else {
			panel->mLbl = WMCreateLabel(panel->win);
			WMMoveWidget(panel->mLbl, 10, 83);
			WMResizeWidget(panel->mLbl, 380, WMFontHeight(WMSystemFontOfSize (scrPtr, 12))*4);
			WMSetLabelText(panel->mLbl, settings->prompt);
			WMSetLabelTextAlignment(panel->mLbl, WACenter);
		}
	}

	WMSetWindowTitle(panel->win, "");

	if ((appicon = WMGetApplicationIconPixmap (scrPtr))) {
		WMSize ais;

		ais = WMGetPixmapSize (appicon);

		panel->iLbl = WMCreateLabel(panel->win);
		WMResizeWidget(panel->iLbl, ais.width, ais.height);
		WMMoveWidget(panel->iLbl, 8 + (64 - ais.width)/2, (75 - ais.height)/2);
		WMSetLabelImage(panel->iLbl, appicon);
		WMSetLabelImagePosition(panel->iLbl, WIPImageOnly);
	}

	if (settings->wintitle) {
		WMSetWindowTitle (panel->win, settings->wintitle);
	}

	if (settings->title) {
		WMFont *largeFont;

		largeFont = WMBoldSystemFontOfSize(scrPtr, 24);

		panel->tLbl = WMCreateLabel(panel->win);
		WMMoveWidget(panel->tLbl, 80, (80 - WMFontHeight(largeFont))/2);
		WMResizeWidget(panel->tLbl, 400 - 70, WMFontHeight(largeFont)+4);
		WMSetLabelText(panel->tLbl, settings->title);
		WMSetLabelTextAlignment(panel->tLbl, WALeft);
		WMSetLabelFont(panel->tLbl, largeFont);

		WMReleaseFont(largeFont);
	}

	/* create divider line */

	panel->line = WMCreateFrame(panel->win);
	WMMoveWidget(panel->line, 0, 80);
	WMResizeWidget(panel->line, 400, 2);
	WMSetFrameRelief(panel->line, WRGroove);

	/* create buttons */
	if (settings->other)
		ow = WMWidthOfString(WMSystemFontOfSize (scrPtr, 12), settings->other,
			     strlen(settings->other));

	if (settings->alt)
		aw = WMWidthOfString(WMSystemFontOfSize (scrPtr, 12), settings->alt,
			     strlen(settings->alt));

	if (settings->def)
		dw = WMWidthOfString(WMSystemFontOfSize (scrPtr, 12), settings->def,
			     strlen(settings->def));

	psize = WMGetPixmapSize (WMGetSystemPixmap (scrPtr, WSIReturnArrow));

	w = dw + (WMGetSystemPixmap (scrPtr, WSIReturnArrow) ? psize.width : 0);
	if (aw > w)
		w = aw;
	if (ow > w)
		w = ow;

	w += 30;
	x = 400;

	if (settings->def) {
		x -= w + 10;
		panel->defBtn = WMCreateCommandButton(panel->win);
		WMSetButtonAction(panel->defBtn, clickPanel, panel);
		WMMoveWidget(panel->defBtn, x, WMWidgetHeight (panel->win)-36);
		WMResizeWidget(panel->defBtn, w, 24);
		WMSetButtonText(panel->defBtn, settings->def);
		WMSetButtonImage(panel->defBtn, WMGetSystemPixmap (scrPtr, WSIReturnArrow));
		WMSetButtonAltImage(panel->defBtn, WMGetSystemPixmap (scrPtr, WSIHighlightedReturnArrow));
		WMSetButtonImagePosition(panel->defBtn, WIPRight);
	}

	if (settings->alt) {
		x -= w + 10;
		panel->altBtn = WMCreateCommandButton(panel->win);
		WMMoveWidget(panel->altBtn, x, WMWidgetHeight (panel->win)-36);
		WMResizeWidget(panel->altBtn, w, 24);
		WMSetButtonAction(panel->altBtn, clickPanel, panel);
		WMSetButtonText(panel->altBtn, settings->alt);
	}

	if (settings->other) {
		x -= w + 10;
		panel->othBtn = WMCreateCommandButton(panel->win);
		WMSetButtonAction(panel->othBtn, clickPanel, panel);
		WMMoveWidget(panel->othBtn, x, WMWidgetHeight (panel->win)-36);
		WMResizeWidget(panel->othBtn, w, 24);
		WMSetButtonText(panel->othBtn, settings->other);
	}

	if (settings->timeout > 0) {
		settings->timerTimeout = WMAddTimerHandler (settings->timeout * 1000, timeoutCB, panel);
	}

	if (settings->reload > 0) {
		settings->timerReload = WMAddTimerHandler (settings->reload * 1000, reloadCB, panel);
	}

	panel->done = 0;

	if (settings->nokeys == False)
		WMCreateEventHandler(WMWidgetView(panel->win), KeyPressMask, pressPanel, panel);

	if (settings->nomouse == False && settings->nobuttons == True)
		WMCreateEventHandler(WMWidgetView(panel->win), ButtonReleaseMask, releasePanel, panel);

	WMMapSubwidgets(panel->win);

	return panel;
}
