
/*
 *  wmessage.app -- wmessage.c [main / startup]
 *
 *  Copyright (c) 1999,2000 Craig Nellist <crn@ozemail.com.au>
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 *  USA.
 */

#include "wmessage.h"
#include "logo.xpm"

void wAbort()
{
	exit(-2);
}

char *ProgName;

void check_args (int argc, char **argv)
{
	int i;
#ifdef MIMIC_XMESSAGE
	int gotmsg = False;
#endif

	settings = wmalloc (sizeof (_settings));
	memset (settings, 0, sizeof (_settings));

	settings->display_name = "";
	settings->timeout = 0;
	settings->reload = 0;
	settings->noicon = False;
	settings->print = False;
	settings->centre = False;
	settings->nokeys = False;
	settings->nobuttons = True;
	settings->nomouse = False;
	settings->warnings = True;
	settings->fixedfont = False;
	settings->nearmouse = False;
	settings->emptyfile = False;
	settings->force = False;

	settings->source = SOURCE_MOUSE;

	if (argc > 1) {
		for (i = 1; i < argc; i++) {

			if (!strncmp(argv[i], "--", 2)) /* eat the first - if there's two there */
				++(argv[i]);

			if (!strcmp(argv[i], "-help") || !strcmp(argv[i], "-h"))
				usage();

			if (!strcmp(argv[i], "-display")) {
				if (++i >= argc) {
					fprintf (stderr, "not enough arguments for %s\n", argv[i-1]);
					exit (-2);
				}
				settings->display_name = argv[i];
			} else if (!strcmp(argv[i], "-timeout")) {
				if (++i >= argc) {
					fprintf (stderr, "not enough arguments for %s\n", argv[i-1]);
					exit (-2);
				}
				settings->timeout = atoi(argv[i]);
			} else if (!strcmp(argv[i], "-reload")) {
				if (++i >= argc) {
					fprintf (stderr, "not enough arguments for %s\n", argv[i-1]);
					exit (-2);
				}
				settings->reload = atoi(argv[i]);
			} else if (!strcmp(argv[i], "-icon")) {
				if (++i >= argc) {
					fprintf (stderr, "not enough arguments for %s\n", argv[i-1]);
					exit (-2);
				}
				settings->icon = argv[i];
			} else if (!strcmp(argv[i], "-file")) {
				if (++i >= argc) {
					fprintf (stderr, "not enough arguments for %s\n", argv[i-1]);
					exit (-2);
				}
				settings->filename = argv[i];
			} else if (!strcmp(argv[i], "-noicon"))
				settings->noicon = True;
			else if (!strcmp(argv[i], "-print"))
				settings->print = True;
			else if (!strcmp(argv[i], "-centre") || !strcmp(argv[i], "-center"))
				settings->centre = True;
			else if (!strcmp(argv[i], "-nearmouse"))
				settings->nearmouse = True;
			else if (!strcmp(argv[i], "-ignore-keys"))
				settings->nokeys = True;
			else if (!strcmp(argv[i], "-ignore-mouse"))
				settings->nomouse = True;
			else if (!strcmp(argv[i], "-fixedfont"))
				settings->fixedfont = True;
			else if (!strcmp(argv[i], "-quiet") || !strcmp(argv[i], "-q"))
				settings->warnings = False;
			else if (!strcmp(argv[i], "-emptyfile"))
				settings->emptyfile = True;
			else if (!strcmp(argv[i], "-force"))
				settings->force = True;
			else if (!strcmp(argv[i], "-d")) {
				if (++i >= argc) {
					fprintf (stderr, "not enough arguments for %s\n", argv[i-1]);
					exit (-2);
				}
				settings->def = argv[i];
				settings->nobuttons = False;
			} else if (!strcmp(argv[i], "-a")) {
				if (++i >= argc) {
					fprintf (stderr, "not enough arguments for %s\n", argv[i-1]);
					exit (-2);
				}
				settings->alt = argv[i];
				settings->nobuttons = False;
			} else if (!strcmp(argv[i], "-o")) {
				if (++i >= argc) {
					fprintf (stderr, "not enough arguments for %s\n", argv[i-1]);
					exit (-2);
				}
				settings->other = argv[i];
				settings->nobuttons = False;
			} else if (!strcmp(argv[i], "-p")) {
				if (++i >= argc) {
					fprintf (stderr, "not enough arguments for %s\n", argv[i-1]);
					exit (-2);
				}
				settings->prompt = argv[i];
			} else if (!strcmp(argv[i], "-w")) {
				if (++i >= argc) {
					fprintf (stderr, "not enough arguments for %s\n", argv[i-1]);
					exit (-2);
				}
				settings->wintitle = argv[i];
			} else if (!strcmp(argv[i], "-t")) {
				if (++i >= argc) {
					fprintf (stderr, "not enough arguments for %s\n", argv[i-1]);
					exit (-2);
				}
				settings->title = argv[i];
			} else {
#ifdef MIMIC_XMESSAGE
				if (gotmsg == True) {
					fprintf (stderr, "Invalid option %s. Run '%s --help' for help.\n", argv[i], ProgName);
					exit(-2);
				}
				settings->prompt = argv[i];
				gotmsg = True;
#else
				fprintf (stderr, "Invalid option %s. Run '%s --help' for help.\n", argv[i], ProgName);
				exit (-2);
#endif
			}
		}
	}

#ifdef MIMIC_XMESSAGE
	if (settings->nobuttons == True) {
		settings->def = "Okay";
		settings->nobuttons = False;
	}

	if (settings->title && !settings->wintitle)
		settings->wintitle = settings->title;
#endif

	if (settings->warnings == True) {
		if (settings->nobuttons == True)
			fprintf (stderr, "No buttons defined. Run '%s --help' for help.\n", ProgName);

		if (settings->nokeys == True && settings->nomouse == True && settings->timeout == 0)
			fprintf (stderr, "Ignoring keys; ignoring mouse; no automatic timeout -- press Ctrl-C to close window.\n");
	}
}

void usage(void)
{
 	fprintf(stderr,
		"%s -- message viewer <%s>\n\n"
		"usage:\n"
#ifdef MIMIC_XMESSAGE
		"\t%s [options] [message...]\n"
#else
		"\t%s [options]\n"
#endif
		"\n"
		"options:\n"
		"  --centre/center\tCentre the message window\n"
		"  --display <display>\tDisplay to open on\n"
		"  --emptyfile\t\tDisplay message in window if file is empty\n"
		"  --file <filename>\tFile to show in window (- for stdin)\n"
		"  --fixedfont\t\tShow file text in fixed font\n"
		"  --force\t\tForce reloading of file if it can't be opened\n"
		"  --help\t\tThis help message\n"
		"  --icon <filename>\tIcon to use in panel\n"
		"  --ignore-keys\t\tIgnore any key presses\n"
		"  --ignore-mouse\tIgnore any mouse clicks\n"
		"  --nearmouse\t\tPosition the message window at the mouse pointer\n"
		"  --noicon\t\tDon't display an icon (overrides --icon)\n"
		"  --print\t\tPrint button text when selected\n"
		"  --quiet (-q)\t\tDon't display warnings\n"
		"  --reload <seconds>\tContinually reload the file after an interval\n"
		"  --timeout <seconds>\tClick the default button after timeout\n"
		"  \n"
		"  -d <str>\tText for the default button (default none)\n"
		"  -a <str>\tText for the alternate button (default none)\n"
		"  -o <str>\tText for the other button (default none)\n"
		"  -p <str>\tPrompt message (default none)\n"
		"  -t <str>\tMessage title (default none)\n"
		"  -w <str>\tWindow title (default none)\n"
		"\n"
		"returns:\n"
		"\tdefault button = 0; alternate = 1; other = -1\n"
		,ProgName, AUTHOR, ProgName
	);
	exit(-2);
}

void setAppIcon (WMScreen *scrPtr, char *path)
{
	WMPixmap *icon;
	RImage *tmp;

	tmp = RLoadImage(WMScreenRContext(scrPtr), path, 0);
	if (!tmp) {
		if (settings->warnings)
			fprintf (stderr, "Couldn't load image file '%s' -- %s", path, RMessageForError(RErrorCode));
	} else {
		icon = WMCreatePixmapFromRImage(scrPtr, tmp, 0);
		RDestroyImage(tmp);
		if (icon) {
			WMSetApplicationIconPixmap(scrPtr, icon);
			WMReleasePixmap(icon);
		} else
			if (settings->warnings == True)
				fprintf (stderr, "Couldn't create icon from image '%s'.\n", path);
	}
}

int main(int argc, char **argv)
{
	WMScreen *scr;
	WMPixmap *pixmap;

	WMInitializeApplication("WMessage", &argc, argv);
	ProgName = argv[0];

	check_args(argc, argv);

	settings->dpy = XOpenDisplay (settings->display_name);

	if (!settings->dpy) {
		fprintf (stderr, "Could not open display. Run '%s --help' for help\n", ProgName);
		exit(-2);
	}

	scr = WMCreateSimpleApplicationScreen(settings->dpy);

	if (settings->noicon == False) {
		if (settings->icon) {
			setAppIcon (scr, settings->icon);
		} else {
			pixmap = WMCreatePixmapFromXPMData(scr, GNUSTEP_XPM);
			/* WMSetApplicationIconImage(scr, pixmap); */
			WMSetApplicationIconPixmap(scr, pixmap);
			WMReleasePixmap(pixmap);
		}
	}

	return (showPanel (scr, settings));
}
